
using System;
using System.Collections;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Threading;
using ClarionSimulation;
using WorldServerLibrary;
using WorldServerLibrary.Model;
using WorldServerLibrary.Exceptions;

namespace ClarionDEMO
{
	class MainClass
	{
		#region properties
		private WorldServer worldServer = null;
        private ClarionAgent agent;
        String creatureId = String.Empty;
        String creatureName = String.Empty;
		double prad = 0;
		#endregion

		#region constructor
		public MainClass() {
			Console.WriteLine ("Clarion Demo V0.4");
			agent = new ClarionAgent();
            agent.OnNewVisualSensorialInformation += new InputVisualSensorialInformationEventHandler(agent_OnNewVisualSensorialInformation);
            agent.OnNewExternalActionSelected += new OutputActionChunkEventHandler(agent_OnNewExternalActionSelected);
			try
            {
                worldServer = new WorldServer("localhost", 4011);
//				worldServer = new WorldServer( "192.168.56.101", 4011 );

                String message = worldServer.Connect();

                if (worldServer != null && worldServer.IsConnected)
                {
                    Console.Out.WriteLine ("[SUCCESS] " + message + "\n");
					worldServer.SendWorldReset();
                    worldServer.NewCreature(400, 200, 0, out creatureId, out creatureName);
					worldServer.SendCreateLeaflet();
                    worldServer.NewBrick(4, 747, 2, 800, 567);
                    worldServer.NewBrick(4, 50, -4, 747, 47);
                    worldServer.NewBrick(4, 49, 562, 796, 599);
                    worldServer.NewBrick(4, -2, 6, 50, 599);

                    if (!String.IsNullOrWhiteSpace(creatureId))
                    {
                        worldServer.SendStartCamera(creatureId);
                        worldServer.SendStartCreature(creatureId);
                    }

                    Console.Out.WriteLine("Creature created with name: " + creatureId + "\n");
                    agent.Run();
					Console.Out.WriteLine("Running Simulation ...\n");
                }
            }
            catch (WorldServerInvalidArgument invalidArtgument)
            {
                Console.Out.WriteLine(String.Format("[ERROR] Invalid Argument: {0}\n", invalidArtgument.Message));
            }
            catch (WorldServerConnectionError serverError)
            {
                Console.Out.WriteLine(String.Format("[ERROR] Is is not possible to connect to server: {0}\n", serverError.Message));
            }
            catch (Exception ex)
            {
                Console.Out.WriteLine(String.Format("[ERROR] Unknown Error: {0}\n", ex.Message));
            }
		}
		#endregion

		#region Methods
		public static void Main (string[] args)	{
			new MainClass();
		}

        IList<Thing> agent_OnNewVisualSensorialInformation()
        {
            IList<Thing> response = null;

            if (worldServer != null && worldServer.IsConnected)
            {
                response = worldServer.SendGetCreatureState(creatureName);
				prad = (Math.PI / 180) * response.First().Pitch;
			    while (prad > Math.PI) prad -= 2 * Math.PI;
			    while (prad < - Math.PI) prad += 2 * Math.PI;

            }

            return response;
        }

        void agent_OnNewExternalActionSelected(ClarionAgentActionType externalAction)
        {   Thread.CurrentThread.CurrentCulture = new CultureInfo("en-US");
            if (worldServer != null && worldServer.IsConnected)
            {
                switch (externalAction)
                {
                    case ClarionAgentActionType.DO_NOTHING:
                        // Do nothing as the own value says
                        break;
                    case ClarionAgentActionType.ROTATE_CLOCKWISE:
					    double newprad = prad + Math.PI/1000;
					    while (newprad > Math.PI) newprad -= 2 * Math.PI;
			            while (newprad < - Math.PI) newprad += 2 * Math.PI;
                        worldServer.SendSetAngle(creatureId, 0, 0, newprad);
                        break;
                    case ClarionAgentActionType.GO_AHEAD:
                        worldServer.SendSetAngle(creatureId, 1, 1, prad);
                        break;
	
				// LFR
				case ClarionAgentActionType.GOTO_ITEM:
					{
						IList< Thing > things = worldServer.SendGetCreatureState( creatureName );
						Creature creature = null;

						for( int i = 0; i < things.Count; ++i )
						{
							switch( things[ i ].CategoryId )
							{
							case Thing.CATEGORY_CREATURE:
								creature = ( Creature ) things[ i ];
								break;

							default:
								break;
							}
						}

						foreach( Thing t in agent_OnNewVisualSensorialInformation( ) )
						{
							if( t.CategoryId == Thing.CATEGORY_NPFOOD || t.CategoryId == Thing.categoryPFOOD )
							{
								Console.WriteLine( "I saw an item at [ " + t.comX + " , " + t.comY + " ]..." );
								worldServer.SendSetGoTo( creatureId, 1, 1, t.comX, t.comY );
							}
							else if( t.CategoryId == Thing.CATEGORY_JEWEL )
							{
								Console.WriteLine( "I saw an item at [ " + t.comX + " , " + t.comY + " ]..." );

								if( creature != null )
								{
									Console.WriteLine( "The item is " + t.Material.Color );

									int iCollectedNumber = creature.getLeafletCollectedNumber( t.Material.Color );
									int iTotalNumber = creature.getLeafletTotalNumber( t.Material.Color );

									if( iCollectedNumber < iTotalNumber )
									{
										Console.WriteLine( "I just have " + iCollectedNumber + " of " + iTotalNumber );	
										worldServer.SendSetGoTo( creatureId, 1, 1, t.comX, t.comY );
									}
									else
									{
										Console.WriteLine( "I don't need this color anymore." );	
									}

									Thread.Sleep( 3000 );
								}		
							}
						}
					}
					break;

				case ClarionAgentActionType.GET_ITEM:

/*
					double dMinDistance = 99999;
					int iIndex = 0;
					IList< Thing > things = worldServer.SendGetCreatureState (creatureName);

					for( int i = 0; i < things.Count; ++i )
					{
						switch( things[ i ].CategoryId )
						{
						case Thing.CATEGORY_JEWEL:
						case Thing.CATEGORY_NPFOOD:
						case Thing.categoryPFOOD:

							if( things[ i ].DistanceToCreature < dMinDistance )
							{
								iIndex = i;
								dMinDistance = things[ i ].DistanceToCreature;
							}	

							break;

						default:
							break;
						}
					}

					if( things[ iIndex ].CategoryId == Thing.CATEGORY_JEWEL )
					{
						Console.WriteLine( "It's close. I can take [ " + things[ iIndex ].Name + " ]..." );
						worldServer.SendSackIt( creatureId, things[ iIndex ].Name );
						Console.WriteLine( "I took [ " + things[ iIndex ].Name + " ]!" );
					}
					else if( things[ iIndex ].CategoryId == Thing.CATEGORY_NPFOOD || things[ iIndex ].CategoryId == Thing.categoryPFOOD )
					{
						Console.WriteLine ("It's close. I can eat [ " + things[ iIndex ].Name + " ]..." );
						worldServer.SendEatIt( creatureId, things[ iIndex ].Name );
						Console.WriteLine( "I ate [ " + things[ iIndex ].Name + " ]!" );
					}
*/

					foreach( Thing t in agent_OnNewVisualSensorialInformation( ) )
					{
						if( t.CategoryId == Thing.CATEGORY_JEWEL )
						{
							Console.WriteLine( "It's close. I can take [ " + t.Name + " ]..." );

							if( worldServer.SendSackIt( creatureId, t.Name ).StartsWith( "Jewel_" ) )
							{
								IList< Thing > things = worldServer.SendGetCreatureState( creatureName );
								Creature creature = null;

								for( int i = 0; i < things.Count; ++i )
								{
									switch( things[ i ].CategoryId )
									{
									case Thing.CATEGORY_CREATURE:
										creature = ( Creature ) things[ i ];
										break;

									default:
										break;
									}
								}

								if( creature != null )
								{
									creature.addColleted( t.Material.Color );
									Console.WriteLine( "I took [ " + t.Name + "( " + t.Material.Color + " ) ]!" );
								}
							}
						}
						else if( t.CategoryId == Thing.CATEGORY_NPFOOD || t.CategoryId == Thing.categoryPFOOD )
						{
							Console.WriteLine ("It's close. I can eat [ " + t.Name + " ]..." );
							worldServer.SendEatIt( creatureId, t.Name );
							Console.WriteLine( "I ate [ " + t.Name + " ]!" );
						}
					}

					break;
                
				default:
                    break;
                }
            }
        }

        #endregion
	}
	
}
